@@ -79,8 +79,8 @@ def parse_command_line(argv: list[str]):
     add("verpatch", help="run `verpatch` on the executables")
     add("light", help="trimmed down build")
     add("sbom", help="record SBOM")
-    add("installer", help="create an EXE installer")
-    add("run", help="run the installer")
+    add("exe", help="create an EXE installer")
+    add("run", help="run the EXE installer")
     add("msi", help="create an MSI installer")
     add("sign", help="sign the EXE and MSI installers")
     add("tests", help="run the unit tests", default=False)
@@ -143,6 +143,9 @@ def get_build_args(args) -> list[str]:
             "win32_tools",
             "docs",
             "qt6_client",
+            "websockets_browser_cookie",
+            "service",
+            "yaml",
         ):
             xpra_args.append(f"--without-{option}")
         xpra_args.append("--with-Os")
@@ -366,9 +369,8 @@ def find_vs_command(name="link") -> str:
 def build_service() -> None:
     step("Compiling system service shim")
     XPRA_SERVICE_EXE = "Xpra-Service.exe"
-    delfile(XPRA_SERVICE_EXE)
     SERVICE_SRC_DIR = os.path.join(os.path.abspath("."), "packaging", "MSWindows", "service")
-    for filename in ("event_log.rc", "event_log.res", "MSG00409.bin", "Xpra-Service.exe"):
+    for filename in ("event_log.rc", "event_log.res", "MSG00409.bin", XPRA_SERVICE_EXE):
         path = os.path.join(SERVICE_SRC_DIR, filename)
         delfile(path)
 
@@ -382,10 +384,10 @@ def build_service() -> None:
                 cwd=SERVICE_SRC_DIR)
     log_command(["g++", "-o", XPRA_SERVICE_EXE, "Xpra-Service.cpp", "-Wno-write-strings"], "service-gcc.log",
                 cwd=SERVICE_SRC_DIR)
-    os.rename(os.path.join(SERVICE_SRC_DIR, XPRA_SERVICE_EXE), XPRA_SERVICE_EXE)
+    os.rename(os.path.join(SERVICE_SRC_DIR, XPRA_SERVICE_EXE), os.path.join(DIST, XPRA_SERVICE_EXE))
 
 
-VersionInfo = namedtuple("VersionInfo", "string,value,revision,full_string,arch_info,extra,padded")
+VersionInfo = namedtuple("VersionInfo", ("string", "value", "revision", "full_string", "arch_info", "extra", "padded"))
 version_info = VersionInfo("invalid", (0, 0), 0, "invalid", "arch", "extra", (0, 0, 0, 0))
 
 
@@ -618,7 +620,7 @@ def delete_libs(*exps: str) -> None:
 def delete_dlls(light: bool) -> None:
     step("Deleting unnecessary DLLs")
     delete_libs(
-        "libjasper*", "lib2to3*", "xdg*", "olefile*", "pygtkcompat*", "jaraco*",
+        "libjasper*", "xdg*", "olefile*", "pygtkcompat*", "jaraco*",
         "p11-kit*", "lz4",
     )
     # remove codecs we don't need:
@@ -677,12 +679,9 @@ def trim_python_libs() -> None:
     step("Removing unnecessary Python modules")
     # remove test bits we don't need:
     delete_libs(
-        "pywin*",
-        "win32com",
         "backports",
         "importlib_resources/compat",
         "importlib_resources/tests",
-        "yaml",
         # no need for headers:
         # "cairo/include"
     )
@@ -767,7 +766,6 @@ def setup_share(light: bool) -> None:
         "share/gstreamer-1.0",
         "share/gst-plugins-base",
         "share/p11-kit",
-        "share/themes/*/gtk-2.0*",
     )
     if light:
         # remove extra bits that take up a lot of space:
@@ -783,16 +781,42 @@ def setup_share(light: bool) -> None:
         rm_empty_dir(f"{DIST}/share/icons")
 
 
-def add_manifests() -> None:
+def add_manifests(light: bool) -> None:
     step("Adding EXE manifests")
     EXES = [
-        "Bug_Report", "Xpra-Launcher", "Xpra", "Xpra_cmd",
-        # these are only included in full builds:
-        "GTK_info", "NativeGUI_info", "Screenshot", "Xpra-Shadow",
+        "Bug_Report", "Xpra-Launcher", "Xpra", "Xpra_cmd", "Configure", "PDFIUM_Print",
     ]
+    if not light:
+        EXES += [
+            "Xpra-Shadow", "Xpra-Proxy_cmd", "Xpra-Proxy",
+            "Python_exec_cmd", "Python_exec_gui",
+            "Xpra-Launcher-Debug",
+            "Config_info",
+            "Python_execfile_cmd", "Python_execfile_gui",
+            "Print", "NVidia_info", "NvFBC_capture", "CUDA_info",
+            "Auth_Dialog",
+            "OpenGL_check",
+            "GTK_info", "NativeGUI_info", "Screenshot",
+            "Version_info", "Network_info", "Keymap_info", "Keyboard_info",
+            "SystemTray_Test", "U2F_Tool",
+            "SQLite_auth_tool", "SQL_auth_tool",
+            "Encoding_info", "Path_info", "Feature_info", "NativeGUI_info",
+            "Xpra_Audio",
+            "GStreamer_info", "Audio_Devices",
+            "gst-launch-1.0", "gst-inspect-1.0",
+            "Webcam_info", "Webcam_Test",
+            "System-Auth-Test", "LDAP-Auth-Test", "LDAP3-Auth-Test",
+            "System-Logon-Test", "Events_Test", "GTK_Keyboard_Test",
+        ]
+    with open('packaging/MSWindows/exe.manifest', 'r') as file:
+        xml_file_content = file.read()
+    zero_padded_version = version_info.padded
+    xml_file_content = xml_file_content.replace("XPRA_ZERO_PADDED_VERSION", zero_padded_version)
+    with open('packaging/MSWindows/exe.manifest.tmp', 'w') as file:
+        file.write(xml_file_content)
     for exe in EXES:
-        if os.path.exists(f"{DIST}/{exe}.exe"):
-            copyfile("packaging/MSWindows/exe.manifest", f"{DIST}/{exe}.exe.manifest")
+        copyfile("packaging/MSWindows/exe.manifest.tmp", f"{DIST}/{exe}.exe.manifest")
+    os.remove('packaging/MSWindows/exe.manifest.tmp')
 
 
 def gen_caches() -> None:
@@ -1146,7 +1170,7 @@ def rec_sbom() -> None:
         package = package_name
         if package.startswith(PACKAGE_PREFIX):
             package = package[len(PACKAGE_PREFIX):]
-        elif os.path.basename(path).startswith(MSYS_DLL_PREFIX):
+        elif os.path.basename(package).startswith(MSYS_DLL_PREFIX):
             # ie: "msys-com_err-1.dll"
             pass  # keep it as it is
         elif package.startswith(MSYS2_PACKAGE_PREFIX):
@@ -1239,8 +1263,8 @@ def create_zip() -> None:
     print(f"{ZIP_FILENAME}: {size}MB")
 
 
-def create_installer(args) -> str:
-    step("Creating the installer using InnoSetup")
+def create_exe(args) -> str:
+    step("Creating the EXE installer using InnoSetup")
     innosetup = find_command("innosetup", "INNOSETUP",
                              f"{PROGRAMFILES}\\Inno Setup 6\\ISCC.exe",
                              f"{PROGRAMFILES_X86}\\Inno Setup 6\\ISCC.exe",
@@ -1288,10 +1312,10 @@ def create_installer(args) -> str:
 
 
 def sign_file(filename: str) -> None:
-    log_command(["signtool.exe", "sign", "/v", "/f", KEY_FILE, "/t", TIMESTAMP_SERVER, filename], "signtool.log")
+    log_command(["signtool.exe", "sign", "/fd", "SHA256", "/v", "/f", KEY_FILE, "/t", TIMESTAMP_SERVER, filename], "signtool.log")
 
 
-def create_msi(installer: str) -> str:
+def create_msi(exe: str) -> str:
     msiwrapper = find_command("msiwrapper", "MSIWRAPPER",
                               f"{PROGRAMFILES}\\MSI Wrapper\\MsiWrapper.exe",
                               f"{PROGRAMFILES_X86}\\MSI Wrapper\\MsiWrapper.exe")
@@ -1299,7 +1323,7 @@ def create_msi(installer: str) -> str:
     # search and replace in the template file:
     subs: dict[str, str] = {
         "CWD": os.getcwd(),
-        "INPUT": installer,
+        "INPUT": exe,
         "OUTPUT": MSI_FILENAME,
         "ZERO_PADDED_VERSION": version_info.padded,
         "FULL_VERSION": version_info.full_string,
@@ -1335,6 +1359,7 @@ def build(args) -> None:
     if args.tests:
         run_tests()
     if args.install:
+        add_manifests(args.light)
         install_exe(args)
 
     if args.fixups:
@@ -1353,7 +1378,6 @@ def build(args) -> None:
     add_numpy(args.numpy)
 
     setup_share(args.light)
-    add_manifests()
     gen_caches()
 
     if args.docs:
@@ -1383,21 +1407,21 @@ def build(args) -> None:
         verpatch()
 
     size = du(DIST) // 1024 // 1024
-    print(f"{size}MB")
+    print(f"installed size: {size}MB")
     if args.zip:
         create_zip()
-    if args.installer:
-        installer = create_installer(args)
-        if not os.path.exists(installer):
-            raise RuntimeError("failed to create installer")
+    if args.exe:
+        exe = create_exe(args)
+        if not os.path.exists(exe):
+            raise RuntimeError(f"failed to create EXE installer {exe!r}")
         if args.sign:
             step("Signing EXE")
-            sign_file(installer)
+            sign_file(exe)
         if args.run:
-            step("Running the new installer")
-            os.system(installer)
+            step(f"Running the new EXE installer {exe!r}")
+            os.system(exe)
         if args.msi:
-            msi = create_msi(installer)
+            msi = create_msi(exe)
             if not os.path.exists(msi):
                 raise RuntimeError("failed to create msi")
             if args.sign:
@@ -1421,7 +1445,7 @@ def main(argv) -> None:
         elif arg == "zip":
             create_zip()
         elif arg == "exe":
-            create_installer(args)
+            create_exe(args)
         else:
             raise ValueError(f"unknown argument {arg!r}")
     else:
