@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2022-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -14,10 +14,7 @@ from xpra.util.str_fn import hexstr
 from xpra.codecs.image import ImageWrapper
 from xpra.codecs.constants import get_subsampling_divs, get_plane_name
 from xpra.codecs.checks import make_test_image
-from xpra.codecs.video import (
-    getVideoHelper,
-    ALL_VIDEO_ENCODER_OPTIONS, ALL_CSC_MODULE_OPTIONS, ALL_VIDEO_DECODER_OPTIONS,
-)
+from xpra.codecs.video import getVideoHelper
 from xpra.log import Logger, consume_verbose_argv
 
 MAX_DELTA = 7
@@ -120,7 +117,7 @@ class Test_Roundtrip(unittest.TestCase):
 
     def test_all(self):
         vh = getVideoHelper()
-        vh.set_modules(ALL_VIDEO_ENCODER_OPTIONS, ALL_CSC_MODULE_OPTIONS, ALL_VIDEO_DECODER_OPTIONS)
+        vh.enable_all_modules()
         vh.init()
         # info = vh.get_info()
         encodings = vh.get_encodings()
@@ -203,7 +200,12 @@ class Test_Roundtrip(unittest.TestCase):
         out_image = decoder.decompress_image(cdata, typedict(client_options))
         if not out_image:
             raise ValueError("no image")
+        if decoder.get_type() == "nvdec":
+            # uses GPU pycuda DeviceAllocation buffers,
+            # which we can't compare directly
+            return
         out_pixels = out_image.get_pixels()
+
         out_csc = out_image.get_pixel_format()
         md = 0
         if in_csc.startswith("YUV") or in_csc == "NV12":
