@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -9,10 +9,10 @@ from typing import Any
 from collections.abc import Sequence, Callable, Iterable
 
 from xpra.util.objects import typedict, reverse_dict
-from xpra.util.str_fn import Ellipsizer, repr_ellipsized, bytestostr
-from xpra.util.env import envbool
+from xpra.util.str_fn import Ellipsizer, repr_ellipsized, csv
+from xpra.util.env import envbool, IgnoreWarningsContext
 from xpra.os_util import gi_import, OSX, WIN32
-from xpra.common import RESOLUTION_ALIASES, ConnectionMessage
+from xpra.common import RESOLUTION_ALIASES, ConnectionMessage, uniq
 from xpra.client.gtk3.menu_helper import (
     MenuHelper,
     BANDWIDTH_MENU_OPTIONS,
@@ -24,8 +24,9 @@ from xpra.client.gtk3.menu_helper import (
 )
 from xpra.exit_codes import ExitCode
 from xpra.codecs.constants import PREFERRED_ENCODING_ORDER
+from xpra.util.config import unset_config_attributes, update_config_attributes
 from xpra.util.stats import std_unit_dec
-from xpra.client.gui import features
+from xpra.client.base import features
 from xpra.log import Logger
 
 GLib = gi_import("GLib")
@@ -56,6 +57,7 @@ MONITORS_MENU = envbool("XPRA_SHOW_MONITORS_MENU", True)
 WINDOWS_MENU = envbool("XPRA_SHOW_WINDOWS_MENU", True)
 START_MENU = envbool("XPRA_SHOW_START_MENU", True)
 MENU_ICONS = envbool("XPRA_MENU_ICONS", True)
+PREFER_IBUS_LAYOUTS = envbool("XPRA_PREFER_IBUS_LAYOUTS", True)
 
 FULL_LAYOUT_LIST = envbool("XPRA_FULL_LAYOUT_LIST", True)
 
@@ -84,36 +86,41 @@ SERVER_NOT_SUPPORTED = "Not supported by the server"
 # 'auto' is recorded as '' unfortunately:
 GENERIC_ENCODINGS = ("", "auto", "stream", "grayscale")
 
+CONFIG = "91_tray.conf"
+
+
+def update_config(attributes: dict[str, str]) -> None:
+    update_config_attributes(attributes, filename=CONFIG)
+
+
+def unset_config(*names: str) -> None:
+    unset_config_attributes(names, filename=CONFIG)
+
 
 class GTKTrayMenu(MenuHelper):
 
     def setup_menu(self):
-        return self.do_setup_menu(SHOW_CLOSE)
+        log("setup_menu()")
+        return self.do_setup_menu(self.get_menu_items())
 
-    def do_setup_menu(self, show_close):
-        log("setup_menu(%s)", show_close)
+    def do_setup_menu(self, items: Sequence[Gtk.ImageMenuItem | Gtk.MenuItem]):
         menu = Gtk.Menu()
+        for menu_item in items:
+            menu.append(menu_item)
+        menu.connect("deactivate", self.menu_deactivated)
+        menu.show_all()
+        return menu
 
-        def add(menuitem) -> None:
-            if menuitem:
-                menu.append(menuitem)
+    def get_menu_items(self) -> list[Gtk.ImageMenuItem | Gtk.MenuItem]:
+        log("get_menu_items()")
+        items: list[Gtk.ImageMenuItem | Gtk.MenuItem] = []
+
+        def add(item) -> None:
+            if item:
+                items.append(item)
 
-        title_item = None
         if SHOW_TITLE_ITEM:
-            title_item = Gtk.MenuItem()
-            title_item.set_label(self.client.session_name or "Xpra")
-            set_sensitive(title_item, False)
-            add(title_item)
-
-            def set_menu_title(*_args) -> None:
-                # set the real name when available:
-                try:
-                    title = self.client.get_tray_title()
-                except Exception:
-                    title = self.client.session_name or "Xpra"
-                title_item.set_label(title)
-
-            self.after_handshake(set_menu_title)
+            add(self.make_titlemenuitem())
         add(self.make_infomenuitem())
         add(self.make_featuresmenuitem())
         if features.windows and self.client.keyboard_helper:
@@ -135,12 +142,31 @@ class GTKTrayMenu(MenuHelper):
         if features.windows and START_MENU:
             add(self.make_startmenuitem())
         add(self.make_disconnectmenuitem())
-        if show_close:
+        if SHOW_CLOSE:
             add(self.make_closemenuitem())
-        menu.connect("deactivate", self.menu_deactivated)
-        menu.show_all()
-        log("setup_menu(%s) done", show_close)
-        return menu
+        return items
+
+    def is_mmap_enabled(self) -> bool:
+        if not getattr(self.client, "", False):
+            return False
+        mra = self.client.mmap_read_area
+        return mra and mra.enabled and mra.size > 0
+
+    def make_titlemenuitem(self) -> Gtk.MenuItem:
+        title_item = Gtk.MenuItem()
+        title_item.set_label(self.client.session_name or "Xpra")
+        set_sensitive(title_item, False)
+
+        def set_menu_title(*_args) -> None:
+            # set the real name when available:
+            try:
+                title = self.client.get_tray_title()
+            except Exception:
+                title = self.client.session_name or "Xpra"
+            title_item.set_label(title)
+
+        self.after_handshake(set_menu_title)
+        return title_item
 
     def make_infomenuitem(self) -> Gtk.ImageMenuItem:
         info_menu_item = self.menuitem("Information", "information.png")
@@ -591,7 +617,7 @@ class GTKTrayMenu(MenuHelper):
         bandwidth_limit_menu_item.show_all()
 
         def set_bwlimitmenu(*_args) -> None:
-            if self.client.mmap_enabled:
+            if self.is_mmap_enabled():
                 bandwidth_limit_menu_item.set_tooltip_text("memory mapped transfers are in use, "
                                                            "so bandwidth limits are disabled")
                 set_sensitive(bandwidth_limit_menu_item, False)
@@ -657,8 +683,8 @@ class GTKTrayMenu(MenuHelper):
 
         def set_encodingsmenuitem(*args) -> None:
             log("set_encodingsmenuitem%s", args)
-            set_sensitive(encodings, not self.client.mmap_enabled)
-            if self.client.mmap_enabled:
+            set_sensitive(encodings, not self.is_mmap_enabled())
+            if self.is_mmap_enabled():
                 # mmap disables encoding and uses raw rgb24
                 encodings.set_label("Encoding")
                 encodings.set_tooltip_text("memory mapped transfers are in use so picture encoding is disabled")
@@ -667,17 +693,8 @@ class GTKTrayMenu(MenuHelper):
                 encodings.set_submenu(self.encodings_submenu)
 
         self.after_handshake(set_encodingsmenuitem)
-        # FUGLY warning: we want to update the menu if we get an 'encodings' packet,
-        # so we inject our handler:
-        saved_process_encodings = getattr(self.client, "_process_encodings")
-        if saved_process_encodings:
-            def process_encodings(*args) -> None:
-                # pass it on:
-                saved_process_encodings(*args)
-                # re-generate the menu with the correct server properties:
-                GLib.idle_add(set_encodingsmenuitem)
-
-            self.client._process_encodings = process_encodings
+        # callback runs from the main thread:
+        self.client.on_server_setting_changed("encoding", set_encodingsmenuitem)
         return encodings
 
     def get_encoding_options(self) -> tuple[Sequence[str], Sequence[str]]:
@@ -814,18 +831,18 @@ class GTKTrayMenu(MenuHelper):
         if self.quality:
             enc = self.client.encoding
             with_quality = enc in self.client.server_encodings_with_quality or enc in GENERIC_ENCODINGS
-            can_use = with_quality and not self.client.mmap_enabled
+            can_use = with_quality and not self.is_mmap_enabled()
             set_sensitive(self.quality, can_use)
-            if self.client.mmap_enabled:
+            if self.is_mmap_enabled():
                 self.quality.set_tooltip_text("Speed is always 100% with mmap")
                 return
             if not can_use:
-                self.quality.set_tooltip_text("Not supported with %s encoding" % enc)
+                self.quality.set_tooltip_text(f"Not supported with {enc!r} encoding")
                 return
             self.quality.set_tooltip_text("Minimum picture quality")
             # now check if lossless is supported:
             if self.quality.get_submenu():
-                can_lossless = self.client.encoding in self.client.server_encodings_with_lossless_mode
+                can_lossless = enc in self.client.server_encodings_with_lossless_mode
                 for q, item in self.quality.get_submenu().menu_items.items():
                     set_sensitive(item, q < 100 or can_lossless)
 
@@ -866,11 +883,11 @@ class GTKTrayMenu(MenuHelper):
         if self.speed:
             enc = self.client.encoding
             with_speed = enc in self.client.server_encodings_with_speed or enc in GENERIC_ENCODINGS
-            set_sensitive(self.speed, with_speed and not self.client.mmap_enabled)
-            if self.client.mmap_enabled:
+            set_sensitive(self.speed, with_speed and not self.is_mmap_enabled())
+            if self.is_mmap_enabled():
                 self.speed.set_tooltip_text("Quality is always 100% with mmap")
             elif not with_speed:
-                self.speed.set_tooltip_text(f"Not supported with {enc} encoding")
+                self.speed.set_tooltip_text(f"Not supported with {enc!r} encoding")
             else:
                 self.speed.set_tooltip_text("Encoding latency vs size")
 
@@ -1110,7 +1127,7 @@ class GTKTrayMenu(MenuHelper):
                 virt_devices = get_virtual_video_devices()
                 non_virtual = {k: v for k, v in all_video_devices.items() if k not in virt_devices}
                 for device_no, info in non_virtual.items():
-                    label = bytestostr(info.get("card", info.get("device", str(device_no))))
+                    label = str(info.get("card", info.get("device", str(device_no))))
                     item = deviceitem(label, start_webcam, device_no)
                     menu.append(item)
                 if not non_virtual:
@@ -1175,36 +1192,148 @@ class GTKTrayMenu(MenuHelper):
     def make_layoutsmenuitem(self) -> Gtk.ImageMenuItem:
         keyboard = self.menuitem("Layout", "keyboard.png", "Select your keyboard layout")
         set_sensitive(keyboard, False)
-        self.layout_submenu = Gtk.Menu()
-        keyboard.set_submenu(self.layout_submenu)
+        self.keyboard_layout_item = keyboard
+        self.after_handshake(self.populate_keyboard_layouts)
+        return keyboard
 
-        def kbitem(title: str, layout: str, variant: str, active=False) -> Gtk.CheckMenuItem:
-            def set_layout(item) -> None:
-                """ this callback updates the client (and server) if needed """
-                ensure_item_selected(self.layout_submenu, item)
-                layout = item.keyboard_layout
-                variant = item.keyboard_variant
+    def populate_keyboard_layouts(self) -> None:
+        log(f"populate_keyboard_layouts() {PREFER_IBUS_LAYOUTS=}")
+        # by default, use keyboard helper values:
+        self.populate_keyboard_helper_layouts()
+        set_sensitive(self.keyboard_layout_item, True)
+
+        if PREFER_IBUS_LAYOUTS:
+            def got_ibus_layouts(setting: str, ibus_layouts) -> None:
                 kh = self.client.keyboard_helper
-                kh.locked = layout != "Auto"
-                if layout != kh.layout_option or variant != kh.variant_option:
-                    if layout == "Auto":
-                        # re-detect everything:
-                        msg = "keyboard automatic mode"
-                    else:
-                        # use layout specified and send it:
-                        kh.layout_option = layout
-                        kh.variant_option = variant
-                        msg = "new keyboard layout selected"
-                    kh.update()
-                    kh.send_layout()
-                    kh.send_keymap()
-                    log.info(f"{msg}: {kh.layout_str()}")
+                Logger("ibus").debug(f"current layout=%r, got {setting!r}=%s", kh.layout, Ellipsizer(ibus_layouts))
+                if ibus_layouts and kh.layout:
+                    self.populate_ibus_keyboard_layouts(ibus_layouts)
+            self.client.on_server_setting_changed("ibus-layouts", got_ibus_layouts)
 
-            l = self.checkitem(str(title), set_layout, active)
-            l.set_draw_as_radio(True)
-            l.keyboard_layout = layout
-            l.keyboard_variant = variant
-            return l
+    def kbitem(self, title: str, layout: str, variant: str, backend="", name="", active=False) -> Gtk.CheckMenuItem:
+        l = self.checkitem(title, self.set_kbitem_layout, active)
+        l.set_draw_as_radio(True)
+        l.keyboard_name = name
+        l.keyboard_backend = backend
+        l.keyboard_layout = layout
+        l.keyboard_variant = variant
+        return l
+
+    def set_kbitem_layout(self, item, save=True) -> None:
+        """ this callback updates the client (and server) if needed """
+        ensure_item_selected(self.layout_submenu, item)
+        descr = item.get_label()
+        name = item.keyboard_name
+        backend = item.keyboard_backend
+        layout = item.keyboard_layout
+        variant = item.keyboard_variant
+        kh = self.client.keyboard_helper
+        kh.locked = layout != "Auto"
+        if layout != kh.layout_option or variant != kh.variant_option or kh.backend != backend or kh.name != name:
+            kh.backend = backend
+            if layout == "Auto":
+                # re-detect everything:
+                msg = "keyboard automatic mode"
+                kh.layout_option = ""
+                kh.variant_option = ""
+                kh.backend = ""
+                kh.name = ""
+                if save:
+                    unset_config("keyboard-backend", "keyboard-layout", "keyboard-variant")
+            else:
+                # use layout specified and send it:
+                kh.layout_option = layout
+                kh.variant_option = variant
+                kh.backend = backend
+                kh.name = name
+                msg = "new keyboard layout selected"
+                if save:
+                    update_config({
+                        "# description": descr,         # ie: "English (UK, extended, Windows)"
+                        "# keyboard name": name,        # ie: "xkb:gb:extd:eng"
+                        "keyboard-backend": backend,
+                        "keyboard-layout": layout,
+                        "keyboard-variant": variant,
+                    })
+            kh.update()
+            kh.send_layout()
+            log.info(f"{msg}: {descr!r}")
+            if not backend:
+                kh.send_keymap()
+
+    def populate_ibus_keyboard_layouts(self, ibus_layouts: dict) -> None:
+        self.layout_submenu = Gtk.Menu()
+        self.keyboard_layout_item.set_submenu(self.layout_submenu)
+        kh = self.client.keyboard_helper
+        matches = []
+
+        def engine_item(engine) -> tuple[str, Gtk.CheckMenuItem]:
+            layout = engine.get("layout", "")
+            name = engine.get("name", layout)
+            descr = engine.get("description", layout).split("\n")[0]
+            variant = engine.get("variant", "")
+            backend = "ibus"
+            active = kh.backend == backend and kh.layout == layout and kh.variant == variant and not matches
+            log(f"{engine=} : {active=}")
+            item = self.kbitem(descr, layout, variant, backend, name, active)
+            return descr, item
+
+        def items_matching(*layouts: str) -> Sequence[Gtk.CheckMenuItem]:
+            # find all the ibus "engines" matching one of these layouts:
+            engines = {}
+            for i, engine in enumerate(ibus_layouts.get("engines", ())):
+                layout = engine.get("layout", "")
+                if not layout or layout not in layouts:
+                    continue
+                rank = engine.get("rank", 0)
+                engines[rank * 65536 + i] = engine
+            items = {}
+            for rank in reversed(sorted(engines)):
+                engine = engines[rank]
+                descr, item = engine_item(engine)
+                if descr not in items:
+                    items[descr] = item
+                    if item.get_active():
+                        matches.append(item)
+            items = dict(sorted(items.items()))
+            return tuple(items.values())
+
+        log(f"current settings: backend={kh.backend!r}, layout={kh.layout!r}, variant={kh.variant!r}")
+
+        # at the top level menu, show layouts matching the layout-option or current layout:
+        layouts = uniq((kh.layout_option or kh.layout).split(","))
+        log(f"unique layouts: {layouts}")
+        for item in items_matching(*layouts):
+            self.layout_submenu.append(item)
+
+        # now add a submenu for each of the other layouts:
+        other_layouts = tuple(layout.strip() for layout in uniq((csv(kh.layouts_option or kh.layouts)).split(","))
+                              if layout.strip() not in layouts)
+        log(f"other layouts: {other_layouts}")
+        from xpra.keyboard.layouts import LAYOUT_NAMES
+        for layout in other_layouts:
+            items = items_matching(layout)
+            log(f"items_matching({layout})={items}")
+            if items:
+                layout_name = LAYOUT_NAMES.get(layout, layout)
+                layout_menu = self.menuitem(layout_name)
+                submenu = Gtk.Menu()
+                for item in items:
+                    submenu.append(item)
+                layout_menu.set_submenu(submenu)
+                self.layout_submenu.append(layout_menu)
+
+        log(f"ibus {matches=}")
+        if matches:
+            self.set_kbitem_layout(matches[0], save=False)
+
+    def populate_keyboard_helper_layouts(self) -> None:
+        self.layout_submenu = Gtk.Menu()
+        self.keyboard_layout_item.set_submenu(self.layout_submenu)
+
+        def disable(message: str) -> None:
+            self.keyboard_layout_item.set_tooltip_text(message)
+            set_sensitive(self.keyboard_layout_item, False)
 
         def keysort(key) -> str:
             c, l = key
@@ -1212,78 +1341,67 @@ class GTKTrayMenu(MenuHelper):
 
         def variants_submenu(layout: str, variants: Iterable[str]) -> None:
             # just show all the variants to choose from this layout
-            default_layout = kbitem(f"{layout} - Default", layout, "", True)
+            default_layout = self.kbitem(f"{layout} - Default", layout, "", active=True)
             self.layout_submenu.append(default_layout)
             for variant in variants:
-                self.layout_submenu.append(kbitem(f"{layout} - {variant}", layout, variant))
+                self.layout_submenu.append(self.kbitem(f"{layout} - {variant}", layout, variant))
 
         kh = self.client.keyboard_helper
-        layout, layouts, variant, variants, _ = kh.get_layout_spec()
-        layout = bytestostr(layout)
-        layouts = tuple(bytestostr(x) for x in layouts)
-        variant = bytestostr(variant or b"")
-        variants = tuple(bytestostr(x) for x in variants)
-        log(f"make_layoutsmenuitem() layout={layout}, layouts={layouts}, variant={variant}, variants={variants}")
+        if not kh:
+            # this can happen when connection fails?
+            return
+        model, layout, layouts, variant, variants, _ = kh.get_layout_spec()
+        log(f"make_layoutsmenuitem() {model=}, {layout=}, {layouts=}, {variant=}, {variants=}")
         if len(layouts) > 1:
-            log("keyboard layouts: %s", ",".join(bytestostr(x) for x in layouts))
-
+            log("keyboard layouts: %s", ",".join(layouts))
             # log after removing dupes:
-
-            def uniq(seq: Iterable) -> list:
-                seen = set()
-                return [x for x in seq if not (x in seen or seen.add(x))]
-
-            log("keyboard layouts: %s", ",".join(bytestostr(x) for x in uniq(layouts)))
-            auto = kbitem("Auto", "Auto", "", True)
+            log("keyboard layouts: %s", ",".join(uniq(layouts)))
+            auto = self.kbitem("Auto", "Auto", "", active=True)
             self.layout_submenu.append(auto)
             if layout:
-                self.layout_submenu.append(kbitem(layout, layout, ""))
+                self.layout_submenu.append(self.kbitem(layout, layout, ""))
             if variants:
                 for v in variants:
-                    self.layout_submenu.append(kbitem(f"{layout} - {v}", layout, v))
+                    self.layout_submenu.append(self.kbitem(f"{layout} - {v}", layout, v))
             for uq_l in uniq(layouts):
                 if uq_l != layout:
-                    self.layout_submenu.append(kbitem(uq_l, uq_l, ""))
-        elif layout and len(variants) > 1:
+                    self.layout_submenu.append(self.kbitem(uq_l, uq_l, ""))
+            return
+        if layout and len(variants) > 1:
             variants_submenu(layout, variants)
-        elif layout or kh.query_struct:
+            return
+        if layout or kh.query_struct:
             khl = layout or kh.query_struct.get("layout", "")
             from xpra.keyboard.layouts import LAYOUT_VARIANTS
             variants = LAYOUT_VARIANTS.get(khl) if khl else ()
             if variants:
                 variants_submenu(khl, variants)
             else:
-                if khl:
-                    keyboard.set_tooltip_text(f"Detected {khl!r}")
-                set_sensitive(keyboard, False)
-                return keyboard
-        elif not FULL_LAYOUT_LIST:
-            keyboard.set_tooltip_text("No keyboard layouts detected")
-            set_sensitive(keyboard, False)
-            return keyboard
-        else:
-            from xpra.keyboard.layouts import X11_LAYOUTS
-            # show all options to choose from:
-            sorted_keys = list(X11_LAYOUTS.keys())
-            sorted_keys.sort(key=keysort)
-            for key in sorted_keys:
-                country, language = key
-                layout, variants = X11_LAYOUTS.get(key)
-                name = f"{country} - {language}"
-                if len(variants) > 1:
-                    # sub-menu for each variant:
-                    variant = self.menuitem(name, tooltip=layout)
-                    variant_submenu = Gtk.Menu()
-                    variant.set_submenu(variant_submenu)
-                    self.layout_submenu.append(variant)
-                    variant_submenu.append(kbitem(f"{layout} - Default", layout, ""))
-                    for v in variants:
-                        variant_submenu.append(kbitem(f"{layout} - {v}", layout, v))
-                else:
-                    # no variants:
-                    self.layout_submenu.append(kbitem(name, layout, ""))
-        self.after_handshake(set_sensitive, keyboard, True)
-        return keyboard
+                disable(f"Detected {khl!r}" if khl else "")
+            return
+        if not FULL_LAYOUT_LIST:
+            disable("No keyboard layouts detected")
+            return
+        from xpra.keyboard.layouts import X11_LAYOUTS
+        # show all options to choose from:
+        sorted_keys = list(X11_LAYOUTS.keys())
+        sorted_keys.sort(key=keysort)
+        for key in sorted_keys:
+            country, language = key
+            layout, variants = X11_LAYOUTS.get(key)
+            name = f"{country} - {language}"
+            if len(variants) > 1:
+                # sub-menu for each variant:
+                variant = self.menuitem(name, tooltip=layout)
+                variant_submenu = Gtk.Menu()
+                variant.set_submenu(variant_submenu)
+                self.layout_submenu.append(variant)
+                variant_submenu.append(self.kbitem(f"{layout} - Default", layout, ""))
+                for v in variants:
+                    variant_submenu.append(self.kbitem(f"{layout} - {v}", layout, v))
+            else:
+                # no variants:
+                self.layout_submenu.append(self.kbitem(name, layout, ""))
 
     def make_monitorsmenuitem(self) -> Gtk.ImageMenuItem:
         monitors_menu_item = self.handshake_menuitem("Monitors", "display.png")
@@ -1486,18 +1604,6 @@ class GTKTrayMenu(MenuHelper):
         self.transfers = self.menuitem("Transfers", "transfer.png",
                                        "Files and URLs forwarding",
                                        self.client.show_ask_data_dialog)
-
-        def enable_transfers(*args) -> None:
-            log("enable_transfers%s ask=%s", args, ())
-            has_ask = any((
-                self.client.remote_file_transfer_ask,
-                self.client.remote_printing_ask,
-                self.client.remote_open_files_ask,
-                self.client.remote_open_url_ask,
-            ))
-            set_sensitive(self.transfers, has_ask)
-
-        self.after_handshake(enable_transfers)
         return self.transfers
 
     def make_uploadmenuitem(self) -> Gtk.ImageMenuItem:
@@ -1529,7 +1635,7 @@ class GTKTrayMenu(MenuHelper):
             elif not remote_send_file:
                 self.download.set_tooltip_text("'send-file' subcommand is not supported by the server")
             else:
-                self.download.set_tooltip_text("Send a file to the server")
+                self.download.set_tooltip_text("Download a file from the server")
 
         self.after_handshake(enable_download)
         return self.download
@@ -1565,9 +1671,9 @@ class GTKTrayMenu(MenuHelper):
             else:
                 messages.append("Shutting down this session may result in data loss,")
             messages.append("are you sure you want to proceed?")
-            dialog = Gtk.MessageDialog(None, 0,
-                                       Gtk.MessageType.QUESTION, Gtk.ButtonsType.NONE,
-                                       "\n".join(messages))
+            dialog = Gtk.MessageDialog(transient_for=None, flags=0,
+                                       message_type=Gtk.MessageType.QUESTION, buttons=Gtk.ButtonsType.NONE,
+                                       text="\n".join(messages))
             dialog.add_button(Gtk.STOCK_CANCEL, 0)
             SHUTDOWN = 1
             dialog.add_button("Shutdown", SHUTDOWN)
@@ -1612,15 +1718,13 @@ class GTKTrayMenu(MenuHelper):
             start_menu_item.set_submenu(menu)
             start_menu_item.set_tooltip_text(None)
 
+        def on_xdg_menu_changed(setting, value):
+            log("on_xdg_menu_changed(%s, %s)", setting, repr_ellipsized(str(value)))
+            update_menu_data()
+        self.client.on_server_setting_changed("xdg-menu", on_xdg_menu_changed)
+
         def start_menu_init() -> None:
             update_menu_data()
-
-            def on_xdg_menu_changed(setting, value):
-                log("on_xdg_menu_changed(%s, %s)", setting, repr_ellipsized(str(value)))
-                update_menu_data()
-
-            self.client.on_server_setting_changed("xdg-menu", on_xdg_menu_changed)
-
         self.after_handshake(start_menu_init)
         return start_menu_item
 
@@ -1658,7 +1762,8 @@ class GTKTrayMenu(MenuHelper):
         if icondata:
             image = get_appimage(title, icondata, self.menu_icon_size)
             if image:
-                smi.set_image(image)
+                with IgnoreWarningsContext():
+                    smi.set_image(image)
         return smi
 
     def make_applaunch_menu_item(self, app_name: str, command_props: typedict) -> Gtk.ImageMenuItem:
