@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2014-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2014 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -154,14 +154,6 @@ def get_info() -> dict[str, Any]:
     return {"version": get_version()}
 
 
-def init_module() -> None:
-    log("gstreamer.init_module()")
-
-
-def cleanup_module() -> None:
-    log("gstreamer.cleanup_module()")
-
-
 IDENTICAL_PIXEL_FORMATS = (
     "NV12",
     "RGBA", "BGRA", "ARGB", "ABGR",
