@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2017-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2017 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -17,7 +17,7 @@ from xpra.exit_codes import ExitCode
 from xpra.net.common import DEFAULT_PORT, AUTO_ABSTRACT_SOCKET, ABSTRACT_SOCKET_PREFIX
 from xpra.net.bytestreams import set_socket_timeout, pretty_socket, SocketConnection, SOCKET_TIMEOUT
 from xpra.os_util import getuid, get_username_for_uid, get_groups, get_group_id, gi_import, WIN32, OSX, POSIX
-from xpra.util.io import path_permission_info, umask_context
+from xpra.util.io import path_permission_info, umask_context, is_writable
 from xpra.util.str_fn import csv, memoryview_to_bytes
 from xpra.util.parsing import parse_simple_dict
 from xpra.util.env import envint, envbool, SilenceWarningsContext
@@ -144,6 +144,8 @@ def hosts(host_str: str) -> list[str]:
         if socket.has_dualstack_ipv6():
             # IPv6 will also listen for IPv4:
             return ["::"]
+        if not socket.has_ipv6:
+            return ["0.0.0.0"]
         # no dual stack, so we have to listen on both IPv4 and IPv6 explicitly:
         return ["0.0.0.0", "::"]
     return [host_str]
@@ -215,7 +217,7 @@ def accept_connection(socktype: str, listener, timeout=None, socket_options=None
     sock.settimeout(timeout)
     sockname = sock.getsockname()
     conn = SocketConnection(sock, sockname, address, peername, socktype, None, socket_options)
-    log("accept_connection(%s, %s, %s)=%s", listener, socktype, timeout, conn)
+    log("accept_connection%s=%s", (listener, socktype, timeout, socket_options), conn)
     return conn
 
 
@@ -305,8 +307,7 @@ def looks_like_xpra_packet(data: bytes) -> bool:
     _, protocol_flags, compression_level, packet_index, data_size = unpack_header(header)
     # this normally used on the first packet, so the packet index should be 0,
     # and I don't think we can make packets smaller than 8 bytes,
-    # even with packet name aliases and rencode
-    # (and aliases should not be defined for the initial packet anyway)
+    # even with rencode
     if packet_index != 0:
         return False
     if data_size < 8 or data_size >= 256 * 1024 * 1024:
@@ -342,7 +343,9 @@ def guess_packet_type(buf: SizedBuffer) -> str:
     if data[:4] == b"RFB ":
         return "vnc"
     if len(data) >= 7 and data[:2] == b"\x03\x00":
-        return "rdp"
+        size = data[2] * 256 + data[3]
+        if len(data) >= size:
+            return "rdp"
     line1 = bytes(data).splitlines()[0]
     if line1.find(b"HTTP/") > 0 or line1.split(b" ")[0] in (b"GET", b"POST"):
         return "http"
@@ -359,6 +362,7 @@ def create_sockets(opts, error_cb: Callable, retry: int = 0,
     bind_ws = parse_bind_ip(opts.bind_ws, 80)
     bind_wss = parse_bind_ip(opts.bind_wss, 443)
     bind_rfb = parse_bind_ip(opts.bind_rfb, 5900)
+    bind_rdp = parse_bind_ip(opts.bind_rdp, 3389)
     bind_quic = parse_bind_ip(opts.bind_quic, 14500)
     bind_vsock = parse_bind_vsock(opts.bind_vsock)
 
@@ -391,6 +395,7 @@ def create_sockets(opts, error_cb: Callable, retry: int = 0,
         "ws": bind_ws,
         "wss": bind_wss,
         "rfb": bind_rfb,
+        "rdp": bind_rdp,
     }.items():
         log("setting up %s sockets: %s", socktype, csv(defs.items()))
         for (host, iport), options in defs.items():
@@ -479,7 +484,7 @@ def setup_tcp_socket(host: str, iport: int, socktype: str = "tcp") \
         raise InitExit(ExitCode.SOCKET_CREATION_ERROR,
                        f"failed to setup {socktype} socket on {host}:{iport} {e}") from None
 
-    def cleanup_tcp_socket():
+    def cleanup_tcp_socket() -> None:
         log.info("closing %s socket %s", socktype.lower(), pretty_socket((host, iport)))
         try:
             tcp_socket.close()
@@ -549,7 +554,7 @@ def setup_udp_socket(host: str, iport: int, socktype: str) -> tuple[str, socket.
     return socktype, udp_socket, (host, iport), cleanup_udp_socket
 
 
-def parse_bind_ip(bind_ip: list[str], default_port: int = DEFAULT_PORT) -> dict[tuple[str, int], dict]:
+def parse_bind_ip(bind_ip: list[str], default_port: int = DEFAULT_PORT) -> dict[tuple[str, int], dict[str, Any]]:
     ip_sockets: dict[tuple[str, int], dict] = {}
     if bind_ip:
         for spec in bind_ip:
@@ -620,7 +625,7 @@ def parse_bind_vsock(bind_vsock: list[str]) -> dict[tuple[int, int], dict]:
 def setup_sd_listen_socket(stype: str, sock, addr) -> tuple[str, socket.socket, Any, Callable]:
     log = get_network_logger()
 
-    def cleanup_sd_listen_socket():
+    def cleanup_sd_listen_socket() -> None:
         log.info(f"closing sd listen socket {pretty_socket(addr)}")
         try:
             sock.close()
@@ -651,6 +656,14 @@ def normalize_local_display_name(local_display_name: str) -> str:
     return local_display_name
 
 
+def can_use_socket_path(path: str, uid: int, gid: int) -> bool:
+    sockdir = os.path.dirname(path)
+    if os.path.exists(sockdir):
+        return is_writable(sockdir, uid, gid)
+    # recurse until we find a directory:
+    return can_use_socket_path(sockdir, uid, gid)
+
+
 def setup_local_sockets(bind, socket_dir: str, socket_dirs, session_dir: str,
                         display_name: str, clobber,
                         mmap_group: str = "auto", socket_permissions: str = "600", username: str = "",
@@ -688,13 +701,18 @@ def setup_local_sockets(bind, socket_dir: str, socket_dirs, session_dir: str,
                 options = parse_simple_dict(parts[1])
             if sockpath in ("auto", "noabstract"):
                 assert display_name is not None
+                # norm_socket_paths is typically used for sockets in $HOME/.xpra/
                 for path in dotxpra.norm_socket_paths(display_name):
-                    sockpaths[path] = dict(options)
+                    if can_use_socket_path(path, uid, gid):
+                        sockpaths[path] = dict(options)
+                    else:
+                        log.info(f"insufficient permissions to use socket path {path!r}")
                 if session_dir and not WIN32:
                     path = os.path.join(session_dir, "socket")
                     sockpaths[path] = dict(options)
                 if sockpath != "noabstract" and AUTO_ABSTRACT_SOCKET:
-                    path = "@" + ABSTRACT_SOCKET_PREFIX + strip_display_prefix(display_name)
+                    sane_display_name = strip_display_prefix(display_name).replace(":", "-").replace(".", "_")
+                    path = "@" + ABSTRACT_SOCKET_PREFIX + sane_display_name
                     abs_options = dict(options)
                     if "auth" not in options:
                         abs_options["auth"] = ABSTRACT_SOCKET_AUTH
