@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (c) 2019-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -30,7 +30,7 @@ def main():
     freeze_support()
 
     os.environ["XPRA_REDIRECT_OUTPUT"] = "1"
-    # os.environ["XPRA_LOG_FILENAME"] = "E:\\Shadow.log"
+    os.environ["XPRA_LOG_FILENAME"] = "Shadow.log"
     # os.environ["XPRA_ALL_DEBUG"] = "1"
     # os.environ["XPRA_NAMED_PIPE_UNRESTRICTED"] = "1"
     from xpra.platform import init, set_default_name
