@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -9,19 +9,23 @@ import os
 from typing import Any
 from collections.abc import Callable
 
+from xpra.os_util import gi_import
 from xpra.util.str_fn import strtobytes, bytestostr, hexstr
 from xpra.util.objects import typedict
 from xpra.util.env import envbool
 from xpra.gtk.error import xswallow, xsync, xlog
 from xpra.scripts.config import str_to_bool
-from xpra.server import EXITING_CODE
+from xpra.server import ServerExitMode
 from xpra.common import SYNC_ICC
+from xpra.server import features
 from xpra.x11.server.core import X11ServerCore, XTestPointerDevice
 from xpra.x11.bindings.keyboard import X11KeyboardBindings
-from xpra.x11.gtk_x11.prop import prop_set, prop_del
-from xpra.x11.xsettings_prop import XSettingsType, BLACKLISTED_XSETTINGS
+from xpra.x11.gtk.prop import prop_set, prop_del
+from xpra.x11.xsettings_prop import XSettingsType, BLOCKLISTED_XSETTINGS
 from xpra.log import Logger
 
+GLib = gi_import("GLib")
+
 log = Logger("x11", "server")
 mouselog = Logger("x11", "server", "mouse")
 screenlog = Logger("server", "screen")
@@ -32,12 +36,12 @@ X11Keyboard = X11KeyboardBindings()
 SCALED_FONT_ANTIALIAS = envbool("XPRA_SCALED_FONT_ANTIALIAS", False)
 
 
-def root_prop_set(prop_name: str, prop_type, value) -> None:
+def root_prop_set(prop_name: str, prop_type: list | tuple | str, value) -> None:
     # pylint: disable=import-outside-toplevel
     prop_set(X11Keyboard.get_root_xid(), prop_name, prop_type, value)
 
 
-def root_prop_del(prop_name: str):
+def root_prop_del(prop_name: str) -> None:
     prop_del(X11Keyboard.get_root_xid(), prop_name)
 
 
@@ -68,10 +72,6 @@ class X11ServerBase(X11ServerCore):
         super().__init__()
         self._default_xsettings: tuple[int, list[tuple]] = (0, [])
         self._settings: dict[str, Any] = {}
-        self.double_click_time: int = 0
-        self.double_click_distance: int = 0
-        self.dpi: int = 0
-        self.default_dpi: int = 0
         self._xsettings_manager = None
         self._xsettings_enabled: bool = False
         self.display_pid: int = 0
@@ -108,25 +108,24 @@ class X11ServerBase(X11ServerCore):
 
     def kill_display(self) -> None:
         if self.display_pid:
-            if self._upgrading == EXITING_CODE:
-                log.info("exiting: not cleaning up Xvfb")
-            elif self._upgrading:
-                log.info("upgrading: not cleaning up Xvfb")
-            else:
-                from xpra.x11.vfb_util import kill_xvfb
-                kill_xvfb(self.display_pid)
-                self.do_clean_session_files(
-                    "xvfb.pid",
-                    "xauthority",
-                    "Xorg.log",
-                    "Xorg.log.old",
-                    "xorg.conf.d/*"
-                    "xorg.conf.d"
-                )
+            if self._exit_mode in (ServerExitMode.UPGRADE, ServerExitMode.EXIT):
+                action = "exiting" if self._exit_mode == ServerExitMode.EXIT else "upgrading"
+                log.info(f"{action}: not cleaning up Xvfb")
+                return
+            from xpra.x11.vfb_util import kill_xvfb
+            kill_xvfb(self.display_pid)
+            self.do_clean_session_files(
+                "xvfb.pid",
+                "xauthority",
+                "Xorg.log",
+                "Xorg.log.old",
+                "xorg.conf.d/*"
+                "xorg.conf.d"
+            )
 
-    def late_cleanup(self) -> None:
-        super().late_cleanup()
+    def late_cleanup(self, stop=True) -> None:
         self.kill_display()
+        super().late_cleanup(stop)
 
     def configure_best_screen_size(self) -> tuple[int, int]:
         root_w, root_h = super().configure_best_screen_size()
@@ -145,16 +144,16 @@ class X11ServerBase(X11ServerCore):
             # now we can save values on the display
             # (we cannot access bindings until dbus has started up)
 
-            def _save_int(prop_name, intval):
+            def _save_int(prop_name, intval) -> None:
                 root_prop_set(prop_name, "u32", intval)
 
-            def _save_str(prop_name, strval):
+            def _save_str(prop_name, strval) -> None:
                 root_prop_set(prop_name, "latin1", strval)
 
             # DBUS_SESSION_BUS_ADDRESS=unix:abstract=/tmp/dbus-B8CDeWmam9,guid=b77f682bd8b57a5cc02f870556cbe9e9
             # DBUS_SESSION_BUS_PID=11406
             # DBUS_SESSION_BUS_WINDOWID=50331649
-            attributes: list[tuple[str, type, Callable[[str, int | str]]]] = [
+            attributes: list[tuple[str, type, Callable[[str, int | str], None]]] = [
                 ("ADDRESS", str, _save_str),
                 ("PID", int, _save_int),
                 ("WINDOW_ID", int, _save_int),
@@ -213,8 +212,8 @@ class X11ServerBase(X11ServerCore):
         nx, ny = 200, 200
         self.pointer_device.move_pointer(nx, ny)
 
-        def verify_uinput_moved():
-            pos = None  # @UnusedVariable
+        def verify_uinput_moved() -> None:
+            pos = (ox, oy)
             with xswallow:
                 pos = X11Keyboard.query_pointer()
                 mouselog("X11Keyboard.query_pointer=%s", pos)
@@ -225,7 +224,7 @@ class X11ServerBase(X11ServerCore):
                 self.pointer_device = xtest
                 self.input_devices = "xtest"
 
-        self.timeout_add(1000, verify_uinput_moved)
+        GLib.timeout_add(1000, verify_uinput_moved)
 
     def dpi_changed(self) -> None:
         # re-apply the same settings, which will apply the new dpi override to it:
@@ -291,6 +290,8 @@ class X11ServerBase(X11ServerCore):
             self._xsettings_manager.set_settings(v)
 
     def init_all_server_settings(self) -> None:
+        if not features.display:
+            return
         log("init_all_server_settings() dpi=%i, default_dpi=%i", self.dpi, self.default_dpi)
         # almost like update_all, except we use the default_dpi,
         # since this is called before the first client connects
@@ -308,9 +309,16 @@ class X11ServerBase(X11ServerCore):
             }, reset=reset)
 
     def update_server_settings(self, settings=None, reset=False) -> None:
+        if not features.display:
+            return
+        cursor_size = getattr(self, "cursor_size", 0)
+        dpi = getattr(self, "dpi", 0)
+        antialias = getattr(self, "antialias", {})
+        double_click_time = getattr(self, "double_click_time", 0)
+        double_click_distance = getattr(self, "double_click_distance", (-1, -1))
         self.do_update_server_settings(settings or self._settings, reset,
-                                       self.dpi, self.double_click_time, self.double_click_distance,
-                                       self.antialias, self.cursor_size)
+                                       dpi, double_click_time, double_click_distance,
+                                       antialias, cursor_size)
 
     def do_update_server_settings(self, settings, reset=False,
                                   dpi=0, double_click_time=0, double_click_distance=(-1, -1),
@@ -354,8 +362,8 @@ class X11ServerBase(X11ServerCore):
                     if len(parts) != 2:
                         log(f"skipped invalid option: {option!r}")
                         continue
-                    if parts[0] in BLACKLISTED_XSETTINGS:
-                        log(f"skipped blacklisted option: {option!r}")
+                    if parts[0] in BLOCKLISTED_XSETTINGS:
+                        log(f"skipped blocklisted option: {option!r}")
                         continue
                     values[parts[0]] = parts[1]
                 if cursor_size > 0:
@@ -395,18 +403,18 @@ class X11ServerBase(X11ServerCore):
             # (as those may not be present in xsettings on some platforms… like win32 and osx)
             have_override = self.double_click_time > 0 or self.double_click_distance != (-1, -1) or antialias or dpi > 0
             if k == "xsettings-blob" and have_override:
-                # start by removing blacklisted options:
-                def filter_blacklisted():
+                # start by removing blocklisted options:
+                def filter_blocklisted():
                     serial, values = v
                     new_values = []
                     for _t, _n, _v, _s in values:
-                        if bytestostr(_n) in BLACKLISTED_XSETTINGS:
-                            log("skipped blacklisted option %s", (_t, _n, _v, _s))
+                        if bytestostr(_n) in BLOCKLISTED_XSETTINGS:
+                            log("skipped blocklisted option %s", (_t, _n, _v, _s))
                         else:
                             new_values.append((_t, _n, _v, _s))
                     return serial, new_values
 
-                v = filter_blacklisted()
+                v = filter_blocklisted()
 
                 def set_xsettings_value(name, value_type, value):
                     # remove existing one, if any:
@@ -424,7 +432,7 @@ class X11ServerBase(X11ServerCore):
                 if dpi > 0:
                     v = set_xsettings_int("Xft/DPI", dpi * 1024)
                 if double_click_time > 0:
-                    v = set_xsettings_int("Net/DoubleClickTime", self.double_click_time)
+                    v = set_xsettings_int("Net/DoubleClickTime", double_click_time)
                 if antialias:
                     ad = typedict(antialias)
                     v = set_xsettings_int("Xft/Antialias", ad.intget("enabled", -1))
@@ -452,4 +460,4 @@ class X11ServerBase(X11ServerCore):
                     log(f"server_settings: setting {p} to {v}")
                     root_prop_set(p, "latin1", strtobytes(v).decode("latin1"))
                 else:
-                    log.warn(f"Warning: unexpected setting {bytestostr(k)}")
+                    log.warn(f"Warning: unexpected setting {k}")
