@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -9,7 +9,7 @@ import hashlib
 from typing import Any
 
 from xpra.util.objects import typedict
-from xpra.util.str_fn import csv
+from xpra.util.str_fn import csv, Ellipsizer
 from xpra.util.env import envbool
 from xpra.os_util import gi_import
 from xpra.gtk.keymap import get_gtk_keymap, get_default_keymap
@@ -30,6 +30,7 @@ from xpra.x11.bindings.keyboard import X11KeyboardBindings
 from xpra.log import Logger
 
 log = Logger("keyboard")
+verboselog = Logger("keyboard", "verbose")
 
 Gdk = gi_import("Gdk")
 
@@ -60,6 +61,24 @@ def kmlog(keyname: str, msg: str, *args) -> None:
     lfn(msg, *args)
 
 
+def get_levels(mode: bool, shift: bool, group: bool) -> list[int]:
+    levels: list[int] = []
+
+    # try both boolean options, current one first:
+    def pair(bval: bool) -> tuple[int, int]:
+        return int(bval), int(not bval)
+
+    # try to preserve the mode (harder to toggle):
+    for m in pair(mode):
+        # try to preserve shift state:
+        for s in pair(shift):
+            # group is comparatively easier to toggle (one function call):
+            for g in pair(group):
+                level = int(g) * 4 + int(m) * 2 + int(s) * 1
+                levels.append(level)
+    return levels
+
+
 class KeyboardConfig(KeyboardConfigBase):
     def __init__(self):
         super().__init__()
@@ -146,7 +165,8 @@ class KeyboardConfig(KeyboardConfigBase):
         # info["state"] = {
         #    "modifiers" : self.get_current_mask(),
         #    }
-        log("keyboard info: %s", info)
+        log("keyboard info: %s", Ellipsizer(info))
+        verboselog("full keyboard info: %s", info)
         return info
 
     def parse_options(self, props: typedict) -> int:
@@ -177,7 +197,7 @@ class KeyboardConfig(KeyboardConfigBase):
             parse_option(x, keymap_dict.strtupleget, props.strtupleget)
         parse_option("raw", keymap_dict.boolget, props.boolget)
         parse_option("layout_groups", keymap_dict.boolget, props.boolget, True)
-        log("assign_keymap_options(..) modified %s", modded)
+        log("parse_options(..) modified %s", Ellipsizer(modded))
         return len(modded)
 
     def parse_layout(self, props: typedict) -> None:
@@ -206,6 +226,7 @@ class KeyboardConfig(KeyboardConfigBase):
         return "%s/%s/%s/%s" % (self.layout, self.variant, self.options, m.hexdigest())
 
     def compute_modifiers(self) -> None:
+        log("compute_modifiers() raw=%s", self.raw)
         if self.raw:
             with xsync:
                 mod_mappings = X11Keyboard.get_modifier_mappings()
@@ -222,8 +243,10 @@ class KeyboardConfig(KeyboardConfigBase):
                         if v not in l:
                             l.append(v)
         else:
-            log("compute_modifiers() mod_meanings=%s", self.mod_meanings)
-            log("compute_modifiers() keycodes=%s", self.keycodes)
+            verboselog("compute_modifiers() mod_meanings=%s, keycodes=%s",
+                       Ellipsizer(self.mod_meanings), Ellipsizer(self.keycodes))
+            verboselog("compute_modifiers() mod_meanings=%s", self.mod_meanings)
+            verboselog("compute_modifiers() keycodes=%s", self.keycodes)
             if self.mod_meanings:
                 # Unix-like OS provides modifier meanings:
                 self.keynames_for_mod = get_modifiers_from_meanings(self.mod_meanings)
@@ -368,7 +391,7 @@ class KeyboardConfig(KeyboardConfigBase):
                     # if the client does not provide a full native keymap with all the keycodes,
                     # try to preserve the initial server keycodes and translate the client keycodes instead:
                     # (used by non X11 clients like osx,win32 or HTML5)
-                    self.keycode_translation = set_keycode_translation(self.x11_keycodes, self.keycodes)
+                    self.keycode_translation.update(set_keycode_translation(self.x11_keycodes, self.keycodes))
                 else:
                     self.keycode_translation = {}
                 self.add_gtk_keynames()
@@ -385,6 +408,7 @@ class KeyboardConfig(KeyboardConfigBase):
                 self.keycode_translation = {}
                 log("keyboard raw mode, keycode translation left empty")
                 self.compute_modifiers()
+
             self.compute_client_modifier_keycodes()
             log("keyname_for_mod=%s", self.keynames_for_mod)
             clean_keyboard_state()
@@ -395,8 +419,9 @@ class KeyboardConfig(KeyboardConfigBase):
         # add the keynames we find via gtk
         # since we may rely on finding those keynames from the client
         # (used with non-native keymaps)
-        log("add_gtk_keynames() gtk keymap=%s", get_gtk_keymap())
-        for _, keyname, keycode, _, _ in get_gtk_keymap():
+        keymap = get_gtk_keymap()
+        log("add_gtk_keynames() gtk keymap=%s", Ellipsizer(keymap))
+        for _, keyname, keycode, _, _ in keymap:
             if keyname not in self.keycode_translation:
                 self.keycode_translation[keyname] = keycode
                 if keyname in DEBUG_KEYSYMS:
@@ -468,8 +493,8 @@ class KeyboardConfig(KeyboardConfigBase):
             return -1, group
         if self.raw:
             return client_keycode, group
-        if self.query_struct:
-            keycode = self.keycode_translation.get((client_keycode, keyname)) or client_keycode
+        if self.x11_keycodes and client_keycode > 0:
+            keycode = self.keycode_translation.get((client_keycode, keyname), 0) or client_keycode
             kmlog(keyname, "do_get_keycode (%i, %s)=%s (native keymap)", client_keycode, keyname, keycode)
             return keycode, group
         return self.find_matching_keycode(client_keycode, keyname, pressed, modifiers, keyval, keystr, group)
@@ -515,15 +540,7 @@ class KeyboardConfig(KeyboardConfigBase):
                 if name in ("ISO_Level3_Shift", "Mode_switch"):
                     mode = 1
                     break
-        levels = []
-        # try to preserve the mode (harder to toggle):
-        for m in (int(bool(mode)), int(not mode)):
-            # try to preserve shift state:
-            for s in (int(bool(shift)), int(not shift)):
-                # group is comparatively easier to toggle (one function call):
-                for g in (int(bool(group)), int(not group)):
-                    level = int(g) * 4 + int(m) * 2 + int(s) * 1
-                    levels.append(level)
+        levels = get_levels(mode, shift, group)
         kml("will try levels: %s", levels)
         for level in levels:
             keycode = self.keycode_translation.get((keyname, level), -1)
@@ -636,7 +653,7 @@ class KeyboardConfig(KeyboardConfigBase):
                     log("modifier '%s' ignored (in ignored keynames=%s)", modifier, keynames)
                     continue
                 m.add(modifier)
-            log("filtered_modifiers_set(%s)=%s", modifiers, m)
+            verboselog("filtered_modifiers_set(%s)=%s", modifiers, m)
             return m
 
         def change_mask(modifiers: set[str], press: bool, info: str) -> list[str]:
@@ -703,6 +720,7 @@ class KeyboardConfig(KeyboardConfigBase):
                         # maybe doing the full keypress (down+up) worked:
                         new_mask = self.get_current_mask()
                         if (modifier in new_mask) == press:
+                            modkeycode = keycode
                             break
                     log("change_mask(%s) %s modifier '%s' using keycode %s, success: %s",
                         info, modifier_list, modifier, keycode, success)
